@@ -571,6 +571,7 @@ css_parser_parse_value (const gchar *buffer, gint start_pos, gint end_pos, CssVa
 	gint pos = start_pos;
 	gint n = 0;
 	CssValue *list = NULL;
+	CssValue *term = NULL;
 
 	pos = css_parser_parse_whitespace (buffer, pos, end_pos);
 
@@ -578,7 +579,6 @@ css_parser_parse_value (const gchar *buffer, gint start_pos, gint end_pos, CssVa
 		return -1;
 	
 	while (pos < end_pos) {
-		CssValue *term;
 		static gchar list_sep;
 
 		/* If it's the second time we get here, we need to create a value list */
@@ -1044,7 +1044,10 @@ css_parser_parse_declaration (const gchar *buffer, gint start_pos, gint end_pos,
 		result->expr = value;
 
 		*ret_val = result;
-	}	
+	} else {
+		css_value_unref (value);
+	}
+
 	return end_pos;
 }
 
@@ -1141,8 +1144,11 @@ css_parser_parse_ruleset (const gchar *buffer, gint start_pos, gint end_pos, Css
 
 /*	g_print ("Setting n_sel to %d\n", n_sel); */
 	
-	if (ret_val)
+	if (ret_val) {
 		*ret_val = ruleset;
+	} else {
+		css_ruleset_destroy (ruleset);
+	}
 
 	return pos;
 }
@@ -1373,6 +1379,8 @@ css_parser_parse_style_attr (const gchar *buffer, gint len)
 	result = g_new (CssRuleset, 1);
 	result->n_decl = n_decl;
 	result->decl = decl;
+	result->n_sel = 0;
+	result->sel = NULL;
 	
 	return result;
 }
