@@ -24,7 +24,7 @@
 #include "dom-htmlselectelement.h"
 #include "dom-htmloptionelement.h"
 
-GObjectClass *parent_class = NULL;
+static GObjectClass *parent_class = NULL;
 
 DomHTMLFormElement *
 dom_HTMLSelectElement__get_form (DomHTMLSelectElement *select)
